#!/usr/bin/env python

__author__ = 'Liang Huang' 

import sys
logs = sys.stderr

''' P_0.5 grammar:

       program : module
       module : stmt+
       stmt : "print" expr NEWLINE
       expr : decint
            | expr "+" expr
'''

###############################################################################
# Lexer

tokens = ['PRINT','INT','PLUS']  # 3 types of tokens
t_ignore = ' \t'                 # ignore spaces
t_PRINT = r'print'
t_PLUS = r'\+'

def t_INT(t): # t is the current token
    r'(-?)\d+'
    try:
        t.value = int(t.value)
    except ValueError:
        raise Exception("integer value too large", t.value)
        t.value = 0
    return t

def t_newline(t):
    r'\n+'
    t.lexer.lineno += t.value.count("\n")

def t_error(t):
    raise Exception("Illegal character '%s' in line %d pos %d" % (t.value[0], 
                                                                  t.lexer.lineno,
                                                                  t.lexer.lexpos))
    t.lexer.skip(1)

import ply.lex as lex

class MyLexer(object):
    def __init__(self):
        lex.lex() # build regexps
        self.lexer = lex.lexer

    def input(self, stream):
        self.lexer.input(stream)
        self.lexer.lines = stream.split("\n") # internal
        print >> logs, "now lexing..."

    def tokenstr(self, tok):
        return "(%s, %s)" % (tok.type, 
                             ("'%s'" if type(tok.value) is str else '%s') % tok.value)

    def token(self):
        tok = self.lexer.token() # lexer.token
        if not tok:
            print >> logs
            return None
        print >> logs, self.tokenstr(tok),            
        return tok


###############################################################################
# Parser

from ast import *

precedence = (
    ('nonassoc','PRINT'),
    ('left','PLUS')
    )

def p_module(t):
    'module : stmtlist'
    t[0] = Module(body=t[1])

def p_stmtlist(t): # flattens into a list
    '''stmtlist : stmt
                | stmt stmtlist'''
    if len(t) == 2:
        t[0] = [t[1]]
    else:
        t[0] = [t[1]] + t[2]

def p_print_stmt(t):
    'stmt : PRINT expr'
    t[0] = Print(values=[t[2]])

def p_plus_expr(t):
    'expr : expr PLUS expr'
    t[0] = BinOp(left=t[1], right=t[3], op=Add())

def p_int_expr(t):
    'expr : INT'
    t[0] = Num(n=t[1])

def p_error(t):
    raise Exception("Syntax error at '%s'" % t.value)

import ply.yacc as yacc
yacc.yacc()

###############################################################################
# Main

if __name__ == "__main__":

    try:
        mylexer = MyLexer()
        tree = yacc.parse(sys.stdin.read(), lexer=mylexer)
        print >> logs, dump(tree)
        from hw1_ast_liang import generate_c 
        print generate_c(tree)
    except Exception as e:
        print >> logs, e
        import traceback
        traceback.print_exc(file=logs)
